/*!
 * \file ftdi.c
 *
 * \author FTDI
 * \date 20120114
 *
 * Copyright ? 2011 Future Technology Devices International Limited
 * Company Confidential
 *
 * Project: FT12x
 * Module: LPC1114-FT12x Eval
 *
 * Rivision History:
 * 0.1 - initial version
 */

#include "..//include//al.h"
#include "..//..//LPC1114//gpio.h"

#ifdef INFRA_DEBUG_ENABLE
	//int currentDebugLevel = MSG_INFO;
	int currentDebugLevel = MSG_DEBUG;
#endif

#define LEN_DBG_BUF		1000
#define MAX_STR_LEN		100

char Dbg_Fifo[LEN_DBG_BUF];
volatile unsigned int dbgRd=0,dbgWr=0;
char dbgCount;
volatile int intDisNest=0;

extern void GPIOInit( void );


void Dbg_Write(char *string, unsigned int len)
{
	int i=0;
	
	Dbg_Fifo[(dbgWr+ (i++))%LEN_DBG_BUF] = '0' + ((dbgCount / 100)%9);
	Dbg_Fifo[(dbgWr+ (i++))%LEN_DBG_BUF] = '0' + ((dbgCount / 10)%9);
	Dbg_Fifo[(dbgWr+ (i++))%LEN_DBG_BUF] = '0' + (dbgCount%9);
	
	for(i=3;(i<(len+3))&&(i<MAX_STR_LEN+3);i++)
	{
		Dbg_Fifo[(dbgWr+i)%LEN_DBG_BUF] = string[i-3];
	}
	Dbg_Fifo[(dbgWr+i)%LEN_DBG_BUF] = '\n';
	
	dbgWr = (dbgWr + len + 4) % LEN_DBG_BUF;
	dbgCount++;
}

void FT12x_Delay(unsigned long ticks)
{
	volatile unsigned long clk_cnt;
	
	clk_cnt = ticks;
	while(clk_cnt>0)
		clk_cnt--;
}

void FT12x_SetPortOut(void)
{
	GPIOSetDir( 2,  0, 1);
	GPIOSetDir( 2,  1, 1);
	GPIOSetDir( 2,  2, 1);
	GPIOSetDir( 2,  3, 1);
	GPIOSetDir( 2,  4, 1);
	GPIOSetDir( 2,  5, 1);
	GPIOSetDir( 2,  6, 1);
	GPIOSetDir( 2,  7, 1);
}

void FT12x_SetPortIn(void)
{
	GPIOSetDir( 2,  0, 0);
	GPIOSetDir( 2,  1, 0);
	GPIOSetDir( 2,  2, 0);
	GPIOSetDir( 2,  3, 0);
	GPIOSetDir( 2,  4, 0);
	GPIOSetDir( 2,  5, 0);
	GPIOSetDir( 2,  6, 0);
	GPIOSetDir( 2,  7, 0);
}

/* A0=0 -> data
 * A0=1 -> command*/
void FT12x_ReadBuffer(bool A0, uint8 *buffer, uint32 size)
{
	uint32 i;

	FT12x_SetPortIn();

	//set A0
	if(A0)
	{
		LPC_GPIO[2]->MASKED_ACCESS[1<<8] = 0xFFFFFFFF;//only 8th bit is written to, rest unchanged
	}
	else
	{
		LPC_GPIO[2]->MASKED_ACCESS[1<<8] = 0;//only 8th bit is written to, rest unchanged
	}

	for(i=0;i<size;i++)
	{
		//assert RD_N
		LPC_GPIO[2]->MASKED_ACCESS[1<<10] = 0;//only 10th bit is written to, rest unchanged

		//wait xx nano sec
		FT12x_Delay(DELAY_RD_WR);

		//read data
		buffer[i]= (uint8) LPC_GPIO[2]->MASKED_ACCESS[0xFF];

		//deassert RD_N
		LPC_GPIO[2]->MASKED_ACCESS[1<<10] = 0xFFFFFFFF;//only 10th bit is written to, rest unchanged
	}
}

/* A0=0 -> data
 * A0=1 -> command*/
void FT12x_WriteBuffer(bool A0, uint8 *buffer, uint32 size)
{
	uint32 i;

	//take mutex

	FT12x_SetPortOut();

	//set A0
	if(A0)
	{
		LPC_GPIO[2]->MASKED_ACCESS[1<<8] = 0xFFFFFFFF;//only 8th bit is written to, rest unchanged
	}
	else
	{
		LPC_GPIO[2]->MASKED_ACCESS[1<<8] = 0;//only 8th bit is written to, rest unchanged
	}

	for(i=0;i<size;i++)
	{// this loop repeats after 1.672uS... i.e. 598802.39Bytes/S or 4.8Mbps
/* put data on bus */
		LPC_GPIO[2]->MASKED_ACCESS[0xFF] = (uint32) buffer[i];//only first 8 bits are written to, rest unchanged
		//TODO: delay - write data hold time
		FT12x_Delay(DELAY_RD_WR);//20120217

/* strobe WR_N */

		LPC_GPIO[2]->MASKED_ACCESS[1<<9] = 0;//only 9th bit is written to, rest unchanged

		//PULSE WIDTH for WR_N low pulse should be 20nS (minimum)
		// already seeing 344nS on scope without adding any delay here
		//when optimization is -O1, this delay is 62nS
		FT12x_Delay(DELAY_RD_WR);

		LPC_GPIO[2]->MASKED_ACCESS[1<<9] = 0xFFFFFFFF;//only 9th bit is written to, rest unchanged

	}
}

void SetupIOLines(void)
{
	//Set configure pins for different functions
#ifdef ENABLE_SW1_SW2
	LPC_IOCON->R_PIO1_0 = 0xC1; //configure P1.0 as GPIO - SW0
	LPC_IOCON->R_PIO1_1 = 0xC1; //configure P1.1 as GPIO - SW1
#endif
	LPC_IOCON->R_PIO1_2 = 0x81; //configure P1.2 as GPIO - LED1

	//setup direction (1 out, 0 input)
	//P2.0	: P2.7 - I/O (default out)
	GPIOSetDir( 2,  8, 1);//A0	: P2.8  - OUT
	GPIOSetDir( 2,  9, 1);//WR_N	: P2.9  - OOU
	GPIOSetDir( 2, 10, 1);//RD_N	: P2.10 - OUT
	GPIOSetDir( 1, 11, 0);//INT_N : P1.11 - IN(setup and enable interrupt)
#ifdef ENABLE_SW1_SW2
	GPIOSetDir( 1, 0, 0);//SW1 - IN
	GPIOSetDir( 1, 1, 0);//SW2 - IN
#endif
	GPIOSetDir( 1, 2, 1);//LED1 - OUT
	GPIOSetDir( 1, 4, 1);//LED2 - OUT
	GPIOSetDir( 1, 10, 1);//CS_N - OUT
	GPIOSetDir( 3, 2, 0);//RESET - IN

	/*setup initial values*/
	GPIOSetValue( 2,  8, 0); //A0
	GPIOSetValue( 2,  9, 1); //WR_N - keep it high since it is active low
	GPIOSetValue( 2, 10, 1); //RD_N - keep it high since it is active low
	GPIOSetValue( 1, 2, 0);  //LED1 - OFF
	GPIOSetValue( 1, 4, 0);  //LED2 - ON
	GPIOSetValue( 1, 10, 0);  //CS_N - LOW

	/*setup Pullup/down*/

	/* FT12x interrupt pin */
	GPIOSetInterrupt( 1/*PORT1*/, 11/*bitPos*/,0 /* edge*/, 0/*single edge*/, 0 /* 0 rising/falling edge */ );
	//GPIOSetInterrupt( 1/*PORT1*/, 11/*bitPos*/,1 /* level*/, 0/*single edge*/, 0 /* 0:interrupt on LOW level */ );

	/* RESET interrupt pin */
	GPIOSetInterrupt( 3/*PORT3*/, 2/*bitPos*/,0 /*edge*/, 0/*single edge*/, 1 /* 0 rising/ 1 falling edge */ );

#ifdef ENABLE_SW1_SW2
	/* Interupt for key press */
	GPIOSetInterrupt( 1/*PORT1*/, 0/*bitPos*/,0 /* edge*/, 0/*single edge*/, 0 /* 0 rising/falling edge */ );
	GPIOSetInterrupt( 1/*PORT1*/, 1/*bitPos*/,0 /* edge*/, 0/*single edge*/, 0 /* 0 rising/falling edge */ );
#endif

	/*Make sure all other interupts for PORT1are disabled*/
	//GPIOIntDisable( 1, 0);
	//GPIOIntDisable( 1, 1);
	GPIOIntDisable( 1, 2);
	GPIOIntDisable( 1, 3);
	GPIOIntDisable( 1, 4);
	GPIOIntDisable( 1, 5);
	GPIOIntDisable( 1, 6);
	GPIOIntDisable( 1, 7);
	GPIOIntDisable( 1, 8);
	GPIOIntDisable( 1, 9);
	GPIOIntDisable( 1, 10);
	//GPIOIntDisable( 1, 11);

	
	GPIOIntEnable( 1 /*PORT1*/, 11 );
#ifdef ENABLE_SW1_SW2
	GPIOIntEnable( 1 /*PORT1*/, 0 );/*SW0*/
	GPIOIntEnable( 1 /*PORT1*/, 1 );/*SW1*/
#endif
	GPIOIntEnable( 3 /*PORT1*/, 2 );/*Reset*/

	/* Setup interrupt */
	GPIOInit();
}

void outportb( int Port, unsigned char Byte)
{
	uint8 buffer[2];
	buffer[0] = (uint8) Byte;

	FT12x_WriteBuffer((bool)Port, buffer, (uint32) 1);
}

unsigned char inportb( int Port)
{
	uint8 buffer[2];
	unsigned char ch;
	FT12x_ReadBuffer((bool)Port, buffer, (uint32) 1);
	ch = (unsigned char) buffer[0];
	return ch;
}

//ledNo = 0,1
//value = 0,1
void Set_LED(uint8 ledNo, uint8 val)
{
	GPIOSetValue( (uint32)1, (uint32)(ledNo?4:2), (uint32)val);
}

/*reads key press on board*/
unsigned char Read_Key(void)
{
	unsigned char key;
	
	key = (unsigned char) LPC_GPIO[1]->MASKED_ACCESS[0xFF];
	key = ~key;//pressed should be 1
	key &= 0x03;//mask only two bits(SW1 & SW2)
	return key;
}

void disable(void) 
{
	GPIOIntDisable(1,11);
	intDisNest++;
	if(intDisNest>1)
	{
		DBG(MSG_DEBUG, "DISABLE called intDisNest=%u\r\n",(unsigned)intDisNest);
	}
	DBG(MSG_DEBUG, "DISABLE called intDisNest=%u\r\n",(unsigned)intDisNest);
}

void enable(void) 
{
	intDisNest--;
	if(intDisNest<=0)
	{
		intDisNest=0;
		GPIOIntEnable(1,11);
	}
	FT12x_Delay(1);
	DBG(MSG_DEBUG, "ENABLE called intDisNest=%u\r\n",(unsigned)intDisNest);
}

void FT12x_SystemHalt(unsigned int arg)
{
	uint8 ledState=0;
	unsigned long clk_cnt = 0;
	unsigned int temp;

	DBG(MSG_DEBUG, "FT12x_SystemHalt\r\n");
	while(1)
	{
		temp = arg + (unsigned int)ledState;
		Set_LED((uint8)0, 1-ledState);
		Set_LED((uint8)1, ledState);
		Set_LED((uint8)2, 1-ledState);
		Set_LED((uint8)3, ledState);
		Set_LED((uint8)4, 1-ledState);

		clk_cnt = ClockTicks;
		while(ClockTicks < (clk_cnt + 1000));
		{
			if(0 == ledState)
				ledState = 1;
			else
				ledState = 0;
		}
	}
}

