/*!
 * \file isr.c
 *
 * \author FTDI
 * \date 20120114
 *
 * Copyright ? 2011 Future Technology Devices International Limited
 * Company Confidential
 *
 * Project: FT12x
 * Module: LPC1114-FT12x Eval
 *
 * Rivision History:
 * 0.1 - initial version
 */

#include <stdio.h>
#include <string.h>
#include "..//include//ftdi.h"
#include "..//include//ci.h"
#include "..//include//mainloop.h"
#include "..//include//usb100.h"
#include "..//include//al.h"



void bus_reset(void);
void ep0_txdone(void);
void ep0_rxdone(void);
void ep1_txdone(void);
void ep1_rxdone(void);
void ep2_txdone(void);
void ep2_rxdone(void);
void ep3_txdone(void);
void ep3_rxdone(void);
extern void dma_eot(void);

volatile unsigned int BulkInCompleted;//0=uninit, 1=initiated,2,completed
volatile unsigned long isr_pending;

#ifdef SETUP_PACKET_COUNTER
	volatile unsigned int setup_packet_count;
#endif
volatile unsigned char ctrl_packet_type;

volatile unsigned int ack_ep0_in_mainloop;


extern unsigned char buffer1[SIZE_BUFFER1];
extern unsigned short first_go;

unsigned char setup_received;
unsigned char seen_int = 0;

extern volatile unsigned long byteCounterTx;
extern volatile unsigned long byteCounterRx;

volatile unsigned int fsm_state;
volatile unsigned long SizeToTransfer;
volatile unsigned long SizeTransfered;

unsigned int NoOfBulkOuts = 0;
volatile unsigned int DisableBulkOut;

volatile unsigned long LastClockTicks = 0;
volatile unsigned long ClockTicks = 0;

EPPFLAGS bEPPflags;
extern CONTROL_XFER ControlData;
unsigned char buffer[64] = {0};
unsigned long int_EP0_OUT=0,int_EP0_IN=0;
unsigned long level_of_nested_interrupt=0;


void fn_usb_isr()
{
	unsigned int i_st=0;
	
	level_of_nested_interrupt++;
	DISABLE;
	bEPPflags.bits.in_isr = 1;
	i_st = FT12x_ReadInterruptRegister();
	DBG(MSG_DEBUG, "\r\nENTER fn_usb_isr nest=%u i_st=0x%x\r\n",(unsigned)level_of_nested_interrupt,(unsigned)i_st);

	if(i_st & 0xFFFF) {
		if(i_st & FT12X_INT_BUSRESET) {
			bus_reset();
			bEPPflags.bits.bus_reset = 1;
			DBG(MSG_DEBUG, "Bus reset\r\n");
		}
		else {
			if(i_st & FT12X_INT_EOT)
			{
				dma_eot();
				DBG(MSG_DEBUG, "Interrupt: EOT\r\n");
			}

			if(i_st & FT12X_INT_SUSPENDCHANGE)
			{
				bEPPflags.bits.suspend = 1;
				DBG(MSG_DEBUG, "Interrupt: Suspend Change\r\n");
			}

			if(i_st & FT12X_INT_ENDP0IN)
			{
				DBG(MSG_DEBUG, "Interrupt: EP0 IN(%u)\r\n",int_EP0_IN);
currentDebugLevel = MSG_DEBUG;
				ep0_txdone();
currentDebugLevel = MSG_EMERG;
				int_EP0_IN++;
			}
			if(i_st & FT12X_INT_ENDP0OUT)
			{
				DBG(MSG_DEBUG, "Interrupt: EP0 OUT(%u)\r\n",(unsigned)int_EP0_OUT);
currentDebugLevel = MSG_DEBUG;
				ep0_rxdone();
currentDebugLevel = MSG_EMERG;
				int_EP0_OUT++;
			}
			if(i_st & FT12X_INT_ENDP1IN)
			{
				ep1_txdone();
				DBG(MSG_DEBUG, "Interrupt: EP1 IN\r\n");
			}
			if(i_st & FT12X_INT_ENDP1OUT)
			{
				ep1_rxdone();
				DBG(MSG_DEBUG, "Interrupt: EP1 OUT\r\n");
			}

			if(i_st & FT12X_INT_ENDP2IN)
			{
				ep2_txdone();
				DBG(MSG_DEBUG, "Interrupt: EP2 IN\r\n");
			}
			if(i_st & FT12X_INT_ENDP2OUT)
			{
				ep2_rxdone();
				DBG(MSG_DEBUG, "Interrupt: EP2 OUT\r\n");
			}


#ifdef ENABLE_HID_INTF
			if(i_st & FT12X_INT_ENDP3IN)
			{
				ep3_txdone();
				DBG(MSG_DEBUG, "Interrupt: EP3 IN\r\n");
			}
			if(i_st & FT12X_INT_ENDP3OUT)
			{
				ep3_rxdone();
				DBG(MSG_DEBUG, "Interrupt: EP3 OUT\r\n");
			}
#endif
		}
	}

	bEPPflags.bits.in_isr = 0;
	ENABLE;
	level_of_nested_interrupt--;
	DBG(MSG_DEBUG, "EXIT fn_usb_isr PSP=0x%x MSP=0x%x\r\n",(unsigned)__get_PSP,(unsigned)__get_MSP);
}

void bus_reset(void)
{

}

void ep0_rxdone(void)
{
	unsigned char ep_last, i;
	ep_last = FT12x_ReadLastTransactionStatus(0); // Clear interrupt flag
	if (ep_last & FT12X_SETUPPACKET) {

		ControlData.wLength = 0;
		ControlData.wCount = 0;
		if( FT12x_ReadEndpoint(0, (unsigned char *)(&(ControlData.DeviceRequest)),
			sizeof(ControlData.DeviceRequest)) != sizeof(DEVICE_REQUEST) ) {
			FT12x_SetEndpointStatus(0, 1);
			FT12x_SetEndpointStatus(1, 1);
			bEPPflags.bits.control_state = USB_IDLE;
			return;
		}

#ifdef __C51__
		ControlData.DeviceRequest.wValue = SWAP(ControlData.DeviceRequest.wValue);
		ControlData.DeviceRequest.wIndex = SWAP(ControlData.DeviceRequest.wIndex);
		ControlData.DeviceRequest.wLength = SWAP(ControlData.DeviceRequest.wLength);
#endif
		// Acknowledge setup here to unlock in/out endp
		FT12x_AcknowledgeEndpoint(0);
		FT12x_AcknowledgeEndpoint(1);

		ControlData.wLength = ControlData.DeviceRequest.wLength;
		ControlData.wCount = 0;

		if (ControlData.DeviceRequest.bmRequestType & (unsigned char)USB_ENDPOINT_DIRECTION_MASK) {
			bEPPflags.bits.setup_packet = 1;
			bEPPflags.bits.control_state = USB_IDLE;		/* get command */
		}
		else {
			if (ControlData.DeviceRequest.wLength == 0) {
				bEPPflags.bits.setup_packet = 1;
				bEPPflags.bits.control_state = USB_IDLE;		/* set command */
			}
			else {
				if(ControlData.DeviceRequest.wLength > MAX_CONTROLDATA_SIZE) {
					bEPPflags.bits.control_state = USB_IDLE;
					FT12x_SetEndpointStatus(0, 1);
					FT12x_SetEndpointStatus(1, 1);
				}
				else {
					bEPPflags.bits.control_state = USB_RECEIVE;	/* set command with OUT token */
				}
			} // set command with data
		} // else set command
	} // if setup packet

	else if ((bEPPflags.bits.control_state == USB_RECEIVE)){  //|| (ep_last & 0x60)) {
		i =	FT12x_ReadEndpoint(0, ControlData.dataBuffer + ControlData.wCount,
			EP0_PACKET_SIZE);
		ControlData.wCount += i;
		if( i != EP0_PACKET_SIZE || ControlData.wCount >= ControlData.wLength) {
			bEPPflags.bits.setup_packet = 1;
			bEPPflags.bits.control_state = USB_IDLE;
		}
	}

	else {
		bEPPflags.bits.control_state = USB_IDLE;
	}
}

void ep0_txdone(void)
{
	short i = ControlData.wLength - ControlData.wCount;

	unsigned char ep_last;
	DBG(MSG_DEBUG, "ENTER ep0_txdone i=%u\r\n",(unsigned)i);

	ep_last = FT12x_ReadLastTransactionStatus(1); // Clear interrupt flag

	if (bEPPflags.bits.control_state != USB_TRANSMIT)
		return;

	if( i >= EP0_PACKET_SIZE) {
		FT12x_WriteEndpoint(1, ControlData.pData + ControlData.wCount, EP0_PACKET_SIZE);
		ControlData.wCount += EP0_PACKET_SIZE;

		bEPPflags.bits.control_state = USB_TRANSMIT;
	}
	else if( i != 0) {
		FT12x_WriteEndpoint(1, ControlData.pData + ControlData.wCount, i);
		ControlData.wCount += i;

		bEPPflags.bits.control_state = USB_IDLE;
	}
	else if (i == 0){
		FT12x_WriteEndpoint(1, 0, 0); // Send zero packet at the end ???

		bEPPflags.bits.control_state = USB_IDLE;
	}
	DBG(MSG_DEBUG, "EXIT ep0_txdone\r\n");
}

void dma_eot(void)
{
	if(bEPPflags.bits.dma_state == DMA_PENDING)
		bEPPflags.bits.setup_dma = 1;
	else
		bEPPflags.bits.dma_state = DMA_IDLE;
}

void ep1_txdone(void)
{
	FT12x_ReadLastTransactionStatus(3); /* Clear interrupt flag */

}

void ep1_rxdone(void)
{
	FT12x_ReadLastTransactionStatus(2); /* Clear interrupt flag */
}

unsigned char ep2_rx_ready=1;

void ep2_txdone(void)
{
	unsigned char ep_last;

	ep_last = FT12x_ReadLastTransactionStatus(5); /* Clear interrupt flag */
	//ReadyForBulkIn = 1;
	if(BulkInCompleted == 1)
	{
		BulkInCompleted = 2;
	}
	ep2_rx_ready=1;
}

void ep2_rxdone(void)
{
	unsigned char len;
	unsigned char ep_last;

	if(ep2_rx_ready)
	{
		ep2_rx_ready=0;
		ep_last = FT12x_ReadLastTransactionStatus(4); /* Clear interrupt flag */
		len= FT12x_ReadMainEndpoint(tempBuffer);
		DBG(MSG_DEBUG, "ep2_rxdone len=%u\r\n",(unsigned)len);
		len = FT12x_WriteEndpoint(5, tempBuffer, len);
		NoOfBulkOuts++;
	}
}


void ep3_txdone(void)
{
	unsigned char ep_last;
	ep_last = FT12x_ReadLastTransactionStatus(7); /* Clear interrupt flag */	
}

void ep3_rxdone(void)
{
	unsigned char ep_last;
	ep_last = FT12x_ReadLastTransactionStatus(6); /* Clear interrupt flag */
}

